@@ -9,21 +9,21 @@
  ***************************************************************************/
 
 #include "certtree.h"
+
+#include <errno.h>
+#include <wx/imaglist.h>
 #include <map>
 #include <vector>
 #include <algorithm>
+#include <utility>
+#include <iostream>
 
 #include "tqslctrls.h"
 #include "util.h"
 #include "dxcc.h"
 #include "tqslerrno.h"
 #include "tqsltrace.h"
-#include <errno.h>
-#include <wx/imaglist.h>
-
-#include <iostream>
-
-using namespace std;
+#include "wxutil.h"
 
 #include "cert.xpm"
 #include "nocert.xpm"
@@ -31,6 +31,12 @@ using namespace std;
 #include "folder.xpm"
 #include "expired.xpm"
 #include "replaced.xpm"
+
+using std::pair;
+using std::vector;
+using std::map;
+using std::make_pair;
+
 enum {
 	CERT_ICON = 0,
 	NOCERT_ICON = 1,
@@ -49,8 +55,8 @@ BEGIN_EVENT_TABLE(CertTree, wxTreeCtrl)
 END_EVENT_TABLE()
 
 CertTree::CertTree(wxWindow *parent, const wxWindowID id, const wxPoint& pos,
-		const wxSize& size, long style) :
-		wxTreeCtrl(parent, id, pos, size, style), _ncerts(0) {
+		const wxSize& size, long style)
+		: wxTreeCtrl(parent, id, pos, size, style), _ncerts(0) {
 	tqslTrace("CertTree::CertTree");
 	useContextMenu = true;
 	wxBitmap certbm(cert_xpm);
@@ -72,15 +78,15 @@ CertTree::CertTree(wxWindow *parent, const wxWindowID id, const wxPoint& pos,
 
 CertTree::~CertTree() {
 	tqslTrace("CertTree::~CertTree");
+	if (_ncerts >0)
+		tqsl_freeCertificateList(_certs, _ncerts);
+	_ncerts = 0;
 }
 
 CertTreeItemData::~CertTreeItemData() {
-	tqslTrace("CertTreeItemData::~CertTreeItemData");
-	if (_cert)
-		tqsl_freeCertificate(_cert);
 }
 
-typedef pair<wxString,int> certitem;
+typedef pair<wxString, int> certitem;
 typedef vector<certitem> certlist;
 
 static bool
@@ -90,35 +96,33 @@ cl_cmp(const certitem& i1, const certitem& i2) {
 
 int
 CertTree::Build(int flags, const TQSL_PROVIDER *provider) {
-	tqslTrace("CertTree::Build", "flags=%d, provider=%lx", flags, (void *)provider);
-	typedef map<wxString,certlist> issmap;
+	tqslTrace("CertTree::Build", "flags=%d, provider=%lx", flags, static_cast<void *>(const_cast<TQSL_PROVIDER *>(provider)));
+	typedef map<wxString, certlist> issmap;
 	issmap issuers;
 
 	DeleteAllItems();
-	wxTreeItemId rootId = AddRoot(wxT("tQSL Certificates"), FOLDER_ICON);
-	tQSL_Cert *certs;
-	if (tqsl_selectCertificates(&certs, &_ncerts, 0, 0, 0, provider, flags)) {
+	wxTreeItemId rootId = AddRoot(_("tQSL Certificates"), FOLDER_ICON);
+	if (tqsl_selectCertificates(&_certs, &_ncerts, 0, 0, 0, provider, flags)) {
 		if (tQSL_Error != TQSL_SYSTEM_ERROR || tQSL_Errno != ENOENT)
-			displayTQSLError("Error while accessing certificate store");
-		return _ncerts;
+			displayTQSLError(__("Error while accessing certificate store"));
 	}
 	// Separate certs into lists by issuer
-	for (int i = 0; i < _ncerts; i++) {	
+	for (int i = 0; i < _ncerts; i++) {
 		char issname[129];
-		if (tqsl_getCertificateIssuerOrganization(certs[i], issname, sizeof issname)) {
-			displayTQSLError("Error parsing certificate for issuer");
+		if (tqsl_getCertificateIssuerOrganization(_certs[i], issname, sizeof issname)) {
+			displayTQSLError(__("Error parsing certificate for issuer"));
 			return _ncerts;
 		}
 		char callsign[129] = "";
-		if (tqsl_getCertificateCallSign(certs[i], callsign, sizeof callsign - 4)) {
-			displayTQSLError("Error parsing certificate for call sign");
+		if (tqsl_getCertificateCallSign(_certs[i], callsign, sizeof callsign - 4)) {
+			displayTQSLError(__("Error parsing certificate for call sign"));
 			return _ncerts;
 		}
 		strncat(callsign, " - ", sizeof callsign - strlen(callsign)-1);
 		DXCC dxcc;
 		int dxccEntity;
-		if (tqsl_getCertificateDXCCEntity(certs[i], &dxccEntity)) {
-			displayTQSLError("Error parsing certificate for DXCC entity");
+		if (tqsl_getCertificateDXCCEntity(_certs[i], &dxccEntity)) {
+			displayTQSLError(__("Error parsing certificate for DXCC entity"));
 			return _ncerts;
 		}
 		const char *entityName;
@@ -128,7 +132,7 @@ CertTree::Build(int flags, const TQSL_PROVIDER *provider) {
 			entityName = "<UNKNOWN ENTITY>";
 		strncat(callsign, entityName, sizeof callsign - strlen(callsign)-1);
 		callsign[sizeof callsign-1] = 0;
-		issuers[wxString(issname, wxConvLocal)].push_back(make_pair(wxString(callsign, wxConvLocal),i));
+		issuers[wxString::FromUTF8(issname)].push_back(make_pair(wxString::FromUTF8(callsign), i));
 	}
 	// Sort each issuer's list and add items to tree
 	issmap::iterator iss_it;
@@ -140,15 +144,15 @@ CertTree::Build(int flags, const TQSL_PROVIDER *provider) {
 		}
 		certlist& list = iss_it->second;
 		sort(list.begin(), list.end(), cl_cmp);
-		for (int i = 0; i < (int)list.size(); i++) {
-			CertTreeItemData *cert = new CertTreeItemData(certs[list[i].second]);
+		for (int i = 0; i < static_cast<int>(list.size()); i++) {
+			CertTreeItemData *cert = new CertTreeItemData(_certs[list[i].second]);
 			int keyonly = 1;
 			int exp = 0, sup = 0;
 			int icon_type;
-			int keytype = tqsl_getCertificatePrivateKeyType(certs[list[i].second]);
-			tqsl_isCertificateExpired(certs[list[i].second], &exp);
-			tqsl_isCertificateSuperceded(certs[list[i].second], &sup);
-			tqsl_getCertificateKeyOnly(certs[list[i].second], &keyonly);
+			int keytype = tqsl_getCertificatePrivateKeyType(_certs[list[i].second]);
+			tqsl_isCertificateExpired(_certs[list[i].second], &exp);
+			tqsl_isCertificateSuperceded(_certs[list[i].second], &sup);
+			tqsl_getCertificateKeyOnly(_certs[list[i].second], &keyonly);
 			if (keytype == TQSL_PK_TYPE_ERR)
 				icon_type = BROKEN_ICON;
 			else if (keyonly)
@@ -170,7 +174,6 @@ CertTree::Build(int flags, const TQSL_PROVIDER *provider) {
 
 void
 CertTree::SelectCert(tQSL_Cert cert) {
-
 	long serial;
 	if (tqsl_getCertificateSerial(cert, &serial))
 		return;
@@ -209,7 +212,7 @@ void
 CertTree::OnItemActivated(wxTreeEvent& event) {
 	tqslTrace("CertTree::OnItemActivated");
 	wxTreeItemId id = event.GetItem();
-	displayCertProperties((CertTreeItemData *)GetItemData(id), this);
+	displayCertProperties(reinterpret_cast<CertTreeItemData *>(GetItemData(id)), this);
 }
 
 void
@@ -222,11 +225,19 @@ CertTree::OnRightDown(wxMouseEvent& event) {
 		SelectItem(id);
 		tQSL_Cert cert = GetItemData(id)->getCert();
 		int keyonly = 1;
-		if (cert)
+		wxMenu *cm;
+		if (cert) {
 			tqsl_getCertificateKeyOnly(cert, &keyonly);
-		wxMenu *cm = makeCertificateMenu(true, (keyonly != 0));
+			char callsign[129];
+			if (tqsl_getCertificateCallSign(cert, callsign, sizeof callsign)) {
+				cm = makeCertificateMenu(true, (keyonly != 0), NULL);
+			} else {
+				cm = makeCertificateMenu(true, (keyonly != 0), callsign);
+			}
+		} else {
+			cm = makeCertificateMenu(true, (keyonly != 0), NULL);
+		}
 		PopupMenu(cm, event.GetPosition());
 		delete cm;
 	}
 }
-
