@@ -22,6 +22,7 @@
 #include "tqsltrace.h"
 
 #include "util.h"
+#include "wxutil.h"
 
 #include "folder.xpm"
 #include "home.xpm"
@@ -72,7 +73,7 @@ static void
 check_tqsl_error(int rval) {
 	if (rval == 0)
 		return;
-	wxLogError(wxT("%hs"), tqsl_getErrorString());
+	wxLogError(getLocalizedErrorString());
 }
 
 int
@@ -82,7 +83,7 @@ LocTree::Build(int flags, const TQSL_PROVIDER *provider) {
 	locmap callsigns;
 
 	DeleteAllItems();
-	wxTreeItemId rootId = AddRoot(wxT("Station Locations"), FOLDER_ICON);
+	wxTreeItemId rootId = AddRoot(_("Station Locations"), FOLDER_ICON);
         tQSL_Location loc;
         check_tqsl_error(tqsl_initStationLocationCapture(&loc));
         check_tqsl_error(tqsl_getNumStationLocations(loc, &_nloc));
