@@ -9,6 +9,8 @@
  ***************************************************************************/
 
 #include "qsodatadialog.h"
+#include <iostream>
+#include <fstream>
 #include <string>
 #include <vector>
 #include <algorithm>
@@ -24,8 +26,13 @@
 #include "tqsllib.h"
 #include "tqslexcept.h"
 #include "tqsltrace.h"
+#include "wxutil.h"
 
 using std::vector;
+using std::ofstream;
+using std::ios;
+using std::cerr;
+using std::endl;
 
 #define TQSL_ID_LOW 6000
 
@@ -95,7 +102,7 @@ static void set_font(wxWindow *w, wxFont& font) {
 
 class choice {
  public:
-	choice(const wxString& _value, const wxString& _display = wxT(""), int _low = 0, int _high = 0) {
+	explicit choice(const wxString& _value, const wxString& _display = wxT(""), int _low = 0, int _high = 0) {
 		value = _value;
 		display = (_display == wxT("")) ? value : _display;
 		low = _low;
@@ -104,6 +111,7 @@ class choice {
 	wxString value, display;
 	int low, high;
 	bool operator ==(const choice& other) { return other.value == value; }
+	bool operator ==(const wxString& other) { return other == value; }
 };
 
 class valid_list : public vector<choice> {
@@ -150,7 +158,7 @@ init_valid_lists() {
 			return 1;
 		valid_modes.push_back(choice(wxString::FromUTF8(cp)));
 	}
-	valid_rxbands.push_back(choice(wxT(""), wxT("NONE")));
+	valid_rxbands.push_back(choice(wxT(""), _("NONE")));
 	if (tqsl_getNumBand(&count))
 		return 1;
 	for (int i = 0; i < count; i++) {
@@ -176,14 +184,14 @@ init_valid_lists() {
 				hz = "GHz";
 			}
 			if (high == 0)
-				high_s = wxT("UP");
+				high_s = _("UP");
 		}
 		wxString display = wxString::Format(wxT("%hs (%s-%s %hs)"), cp,
                         low_s.c_str(), high_s.c_str(), hz);
 		valid_bands.push_back(choice(wxString::FromUTF8(cp), display, low*scale, high*scale));
 		valid_rxbands.push_back(choice(wxString::FromUTF8(cp), display, low*scale, high*scale));
 	}
-	valid_propmodes.push_back(choice(wxT(""), wxT("NONE")));
+	valid_propmodes.push_back(choice(wxT(""), _("NONE")));
 	if (tqsl_getNumPropagationMode(&count))
 		return 1;
 	for (int i = 0; i < count; i++) {
@@ -191,7 +199,7 @@ init_valid_lists() {
 			return 1;
 		valid_propmodes.push_back(choice(wxString::FromUTF8(cp), wxString::FromUTF8(cp1)));
 	}
-	valid_satellites.push_back(choice(wxT(""), wxT("NONE")));
+	valid_satellites.push_back(choice(wxT(""), _("NONE")));
 	if (tqsl_getNumSatellite(&count))
 		return 1;
 	for (int i = 0; i < count; i++) {
@@ -205,6 +213,8 @@ init_valid_lists() {
 #define LABEL_WIDTH (22*TEXT_WIDTH)
 
 BEGIN_EVENT_TABLE(QSODataDialog, wxDialog)
+	EVT_COMBOBOX(-1, QSODataDialog::OnFieldChanged)
+	EVT_TEXT(-1, QSODataDialog::OnFieldChanged)
 	EVT_BUTTON(QD_OK, QSODataDialog::OnOk)
 	EVT_BUTTON(QD_CANCEL, QSODataDialog::OnCancel)
 	EVT_BUTTON(QD_HELP, QSODataDialog::OnHelp)
@@ -216,8 +226,8 @@ BEGIN_EVENT_TABLE(QSODataDialog, wxDialog)
 	EVT_BUTTON(QD_RECDELETE, QSODataDialog::OnRecDelete)
 END_EVENT_TABLE()
 
-QSODataDialog::QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORecordList *reclist, wxWindowID id, const wxString& title)
-	: wxDialog(parent, id, title), _reclist(reclist), _isend(false), _help(help) {
+QSODataDialog::QSODataDialog(wxWindow *parent, wxString& filename, wxHtmlHelpController *help, QSORecordList *reclist, wxWindowID id, const wxString& title)
+	: wxDialog(parent, id, title), _reclist(reclist), _isend(false), _filename(filename), _help(help) {
 	tqslTrace("QSODataDialog::QSODataDialog", "parent=0x%lx, reclist=0x%lx, id=0x%lx, %s", reinterpret_cast<void *>(parent), reinterpret_cast<void *>(reclist), reinterpret_cast<void *>(id), S(title));
 	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
 	wxFont font = GetFont();
@@ -226,11 +236,14 @@ QSODataDialog::QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORe
 
 #define QD_MARGIN 3
 
-	if (init_valid_lists())
-		throw TQSLException(tqsl_getErrorString());
+	if (init_valid_lists()) {
+		char err[256];
+		strncpy(err, getLocalizedErrorString().ToUTF8(), sizeof err);
+		throw TQSLException(err);
+	}
 	// Call sign
 	wxBoxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
-	sizer->Add(new wxStaticText(this, -1, wxT("Call Sign:"), wxDefaultPosition,
+	sizer->Add(new wxStaticText(this, -1, _("Call Sign:"), wxDefaultPosition,
 		wxSize(LABEL_WIDTH, TEXT_HEIGHT), wxALIGN_RIGHT), 0, wxALL, QD_MARGIN);
 	_call_ctrl = new wxTextCtrl(this, QD_CALL, wxT(""), wxDefaultPosition, wxSize(14*TEXT_WIDTH, TEXT_HEIGHT),
 		0, wxTextValidator(wxFILTER_NONE, &rec._call));
@@ -238,22 +251,24 @@ QSODataDialog::QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORe
 	topsizer->Add(sizer, 0);
 	// Date
 	sizer = new wxBoxSizer(wxHORIZONTAL);
-	sizer->Add(new wxStaticText(this, -1, wxT("UTC Date (YYYY-MM-DD):"), wxDefaultPosition,
+	sizer->Add(new wxStaticText(this, -1, _("UTC Date (YYYY-MM-DD):"), wxDefaultPosition,
 		wxSize(LABEL_WIDTH, TEXT_HEIGHT), wxALIGN_RIGHT), 0, wxALL, QD_MARGIN);
-	sizer->Add(new wxTextCtrl(this, QD_DATE, wxT(""), wxDefaultPosition, wxSize(14*TEXT_WIDTH, TEXT_HEIGHT),
-		0, TQSLDateValidator(&rec._date)), 0, wxALL, QD_MARGIN);
+	_date_ctrl = new wxTextCtrl(this, QD_DATE, wxT(""), wxDefaultPosition, wxSize(14*TEXT_WIDTH, TEXT_HEIGHT),
+		0, TQSLDateValidator(&rec._date));
+	sizer->Add(_date_ctrl, 0, wxALL, QD_MARGIN);
 	topsizer->Add(sizer, 0);
 	// Time
 	sizer = new wxBoxSizer(wxHORIZONTAL);
-	sizer->Add(new wxStaticText(this, -1, wxT("UTC Time (HHMM):"), wxDefaultPosition,
+	sizer->Add(new wxStaticText(this, -1, _("UTC Time (HHMM):"), wxDefaultPosition,
 		wxSize(LABEL_WIDTH, TEXT_HEIGHT), wxALIGN_RIGHT), 0, wxALL, QD_MARGIN);
-	sizer->Add(new wxTextCtrl(this, QD_TIME, wxT(""), wxDefaultPosition, wxSize(14*TEXT_WIDTH, TEXT_HEIGHT),
-		0, TQSLTimeValidator(&rec._time)), 0, wxALL, QD_MARGIN);
+	_time_ctrl = new wxTextCtrl(this, QD_TIME, wxT(""), wxDefaultPosition, wxSize(14*TEXT_WIDTH, TEXT_HEIGHT),
+		0, TQSLTimeValidator(&rec._time));
+	sizer->Add(_time_ctrl, 0, wxALL, QD_MARGIN);
 	topsizer->Add(sizer, 0);
 	// Mode
 	sizer = new wxBoxSizer(wxHORIZONTAL);
 	wxString *choices = valid_modes.GetChoices();
-	sizer->Add(new wxStaticText(this, -1, wxT("Mode:"), wxDefaultPosition,
+	sizer->Add(new wxStaticText(this, -1, _("Mode:"), wxDefaultPosition,
 		wxSize(LABEL_WIDTH, TEXT_HEIGHT), wxALIGN_RIGHT), 0, wxALL, QD_MARGIN);
 	sizer->Add(new wxChoice(this, QD_MODE, wxDefaultPosition, wxDefaultSize,
 		valid_modes.size(), choices, 0, wxGenericValidator(&_mode)), 0, wxALL, QD_MARGIN);
@@ -262,7 +277,7 @@ QSODataDialog::QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORe
 	// Band
 	sizer = new wxBoxSizer(wxHORIZONTAL);
 	choices = valid_bands.GetChoices();
-	sizer->Add(new wxStaticText(this, -1, wxT("Band:"), wxDefaultPosition,
+	sizer->Add(new wxStaticText(this, -1, _("Band:"), wxDefaultPosition,
 		wxSize(LABEL_WIDTH, TEXT_HEIGHT), wxALIGN_RIGHT), 0, wxALL, QD_MARGIN);
 	sizer->Add(new wxChoice(this, QD_BAND, wxDefaultPosition, wxDefaultSize,
 		valid_bands.size(), choices, 0, wxGenericValidator(&_band)), 0, wxALL, QD_MARGIN);
@@ -271,7 +286,7 @@ QSODataDialog::QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORe
 	// RX Band
 	sizer = new wxBoxSizer(wxHORIZONTAL);
 	choices = valid_rxbands.GetChoices();
-	sizer->Add(new wxStaticText(this, -1, wxT("RX Band:"), wxDefaultPosition,
+	sizer->Add(new wxStaticText(this, -1, _("RX Band:"), wxDefaultPosition,
 		wxSize(LABEL_WIDTH, TEXT_HEIGHT), wxALIGN_RIGHT), 0, wxALL, QD_MARGIN);
 	sizer->Add(new wxChoice(this, QD_BAND, wxDefaultPosition, wxDefaultSize,
 		valid_rxbands.size(), choices, 0, wxGenericValidator(&_rxband)), 0, wxALL, QD_MARGIN);
@@ -279,14 +294,14 @@ QSODataDialog::QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORe
 	topsizer->Add(sizer, 0);
 	// Frequency
 	sizer = new wxBoxSizer(wxHORIZONTAL);
-	sizer->Add(new wxStaticText(this, -1, wxT("Frequency (MHz):"), wxDefaultPosition,
+	sizer->Add(new wxStaticText(this, -1, _("Frequency (MHz):"), wxDefaultPosition,
 		wxSize(LABEL_WIDTH, TEXT_HEIGHT), wxALIGN_RIGHT), 0, wxALL, QD_MARGIN);
 	sizer->Add(new wxTextCtrl(this, QD_FREQ, wxT(""), wxDefaultPosition, wxSize(14*TEXT_WIDTH, TEXT_HEIGHT),
 		0, wxTextValidator(wxFILTER_NONE, &rec._freq)), 0, wxALL, QD_MARGIN);
 	topsizer->Add(sizer, 0);
 	// RX Frequency
 	sizer = new wxBoxSizer(wxHORIZONTAL);
-	sizer->Add(new wxStaticText(this, -1, wxT("RX Frequency (MHz):"), wxDefaultPosition,
+	sizer->Add(new wxStaticText(this, -1, _("RX Frequency (MHz):"), wxDefaultPosition,
 		wxSize(LABEL_WIDTH, TEXT_HEIGHT), wxALIGN_RIGHT), 0, wxALL, QD_MARGIN);
 	sizer->Add(new wxTextCtrl(this, QD_RXFREQ, wxT(""), wxDefaultPosition, wxSize(14*TEXT_WIDTH, TEXT_HEIGHT),
 		0, wxTextValidator(wxFILTER_NONE, &rec._rxfreq)), 0, wxALL, QD_MARGIN);
@@ -294,7 +309,7 @@ QSODataDialog::QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORe
 	// Propagation Mode
 	sizer = new wxBoxSizer(wxHORIZONTAL);
 	choices = valid_propmodes.GetChoices();
-	sizer->Add(new wxStaticText(this, -1, wxT("Propagation Mode:"), wxDefaultPosition,
+	sizer->Add(new wxStaticText(this, -1, _("Propagation Mode:"), wxDefaultPosition,
 		wxSize(LABEL_WIDTH, TEXT_HEIGHT), wxALIGN_RIGHT), 0, wxALL, QD_MARGIN);
 	sizer->Add(new wxChoice(this, QD_PROPMODE, wxDefaultPosition, wxDefaultSize,
 		valid_propmodes.size(), choices, 0, wxGenericValidator(&_propmode)), 0, wxALL, QD_MARGIN);
@@ -303,7 +318,7 @@ QSODataDialog::QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORe
 	// Satellite
 	sizer = new wxBoxSizer(wxHORIZONTAL);
 	choices = valid_satellites.GetChoices();
-	sizer->Add(new wxStaticText(this, -1, wxT("Satellite:"), wxDefaultPosition,
+	sizer->Add(new wxStaticText(this, -1, _("Satellite:"), wxDefaultPosition,
 		wxSize(LABEL_WIDTH, TEXT_HEIGHT), wxALIGN_RIGHT), 0, wxALL, QD_MARGIN);
 	sizer->Add(new wxChoice(this, QD_SATELLITE, wxDefaultPosition, wxDefaultSize,
 		valid_satellites.size(), choices, 0, wxGenericValidator(&_satellite)), 0, wxALL, QD_MARGIN);
@@ -311,40 +326,55 @@ QSODataDialog::QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORe
 	topsizer->Add(sizer, 0);
 
 	if (_reclist != 0) {
-		if (_reclist->empty())
+		_newrec = -1;
+		if (_reclist->empty()) {
 			_reclist->push_back(QSORecord());
+			_newrec = 1;
+		}
 		topsizer->Add(new wxStaticLine(this, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 10);
 		_recno_label_ctrl = new wxStaticText(this, QD_RECNOLABEL, wxT(""), wxDefaultPosition,
 			wxSize(20*TEXT_WIDTH, TEXT_HEIGHT), wxST_NO_AUTORESIZE|wxALIGN_CENTER);
 		topsizer->Add(_recno_label_ctrl, 0, wxALIGN_CENTER|wxALL, 5);
 		_recno = 1;
 		sizer = new wxBoxSizer(wxHORIZONTAL);
-		_recbottom_ctrl = new wxBitmapButton(this, QD_RECBOTTOM, wxBitmap(bottom_xpm), wxDefaultPosition, wxSize(18, TEXT_HEIGHT)),
+		// Use a really tiny label font on the buttons, as the labels are there
+		// for accessibility only.
+		wxFont f(1, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
+		_recbottom_ctrl = new wxBitmapButton(this, QD_RECBOTTOM, wxBitmap(bottom_xpm));
+		_recbottom_ctrl->SetLabel(_("Go to the first QSO in this log"));
+		_recbottom_ctrl->SetFont(f);
 		sizer->Add(_recbottom_ctrl, 0, wxTOP|wxBOTTOM, 5);
-		_recdown_ctrl = new wxBitmapButton(this, QD_RECDOWN, wxBitmap(left_xpm), wxDefaultPosition, wxSize(18, TEXT_HEIGHT));
+		_recdown_ctrl = new wxBitmapButton(this, QD_RECDOWN, wxBitmap(left_xpm));
+		_recdown_ctrl->SetLabel(_("Go to the previous QSO in this log"));
+		_recdown_ctrl->SetFont(f);
 		sizer->Add(_recdown_ctrl, 0, wxTOP|wxBOTTOM, 5);
-		_recno_ctrl = new wxTextCtrl(this, QD_RECNO, wxT("1"), wxDefaultPosition,
+		_recno_ctrl = new wxTextCtrl(this, QD_RECNO, wxT(""), wxDefaultPosition,
 			wxSize(4*TEXT_WIDTH, TEXT_HEIGHT));
 		_recno_ctrl->Enable(FALSE);
 		sizer->Add(_recno_ctrl, 0, wxALL, 5);
-		_recup_ctrl = new wxBitmapButton(this, QD_RECUP, wxBitmap(right_xpm), wxDefaultPosition, wxSize(18, TEXT_HEIGHT));
+		_recup_ctrl = new wxBitmapButton(this, QD_RECUP, wxBitmap(right_xpm));
+		_recup_ctrl->SetLabel(_("Go to the next QSO in this log"));
+		_recup_ctrl->SetFont(f);
 		sizer->Add(_recup_ctrl, 0, wxTOP|wxBOTTOM, 5);
-		_rectop_ctrl = new wxBitmapButton(this, QD_RECTOP, wxBitmap(top_xpm), wxDefaultPosition, wxSize(18, TEXT_HEIGHT)),
+		_rectop_ctrl = new wxBitmapButton(this, QD_RECTOP, wxBitmap(top_xpm));
+		_rectop_ctrl->SetLabel(_("Go to the last QSO in this log"));
+		_rectop_ctrl->SetFont(f);
 		sizer->Add(_rectop_ctrl, 0, wxTOP|wxBOTTOM, 5);
 		if (_reclist->size() > 0)
 			rec = *(_reclist->begin());
 		topsizer->Add(sizer, 0, wxALIGN_CENTER);
 		sizer = new wxBoxSizer(wxHORIZONTAL);
-		sizer->Add(new wxButton(this, QD_RECNEW, wxT("Add QSO")), 0, wxALL, 5);
-		sizer->Add(new wxButton(this, QD_RECDELETE, wxT("Delete")), 0, wxALL, 5);
+		_recadd_ctrl = new wxButton(this, QD_RECNEW, _("Add QSO"));
+		sizer->Add(_recadd_ctrl, 0, wxALL, 5);
+		sizer->Add(new wxButton(this, QD_RECDELETE, _("Delete")), 0, wxALL, 5);
 		topsizer->Add(sizer, 0, wxALIGN_CENTER);
 	}
 
 	topsizer->Add(new wxStaticLine(this, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 10);
 	sizer = new wxBoxSizer(wxHORIZONTAL);
-	sizer->Add(new wxButton(this, QD_HELP, wxT("Help")), 0, wxALL, 10);
-	sizer->Add(new wxButton(this, QD_CANCEL, wxT("Cancel")), 0, wxALL, 10);
-	sizer->Add(new wxButton(this, QD_OK, wxT("OK")), 0, wxALL, 10);
+	sizer->Add(new wxButton(this, QD_HELP, _("Help")), 0, wxALL, 10);
+	sizer->Add(new wxButton(this, QD_CANCEL, _("Cancel")), 0, wxALL, 10);
+	sizer->Add(new wxButton(this, QD_OK, _("OK")), 0, wxALL, 10);
 	topsizer->Add(sizer, 0, wxALIGN_CENTER);
 
 	UpdateControls();
@@ -354,12 +384,40 @@ QSODataDialog::QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORe
 	topsizer->Fit(this);
 	topsizer->SetSizeHints(this);
 
+	SetFocus();
 	CentreOnParent();
 }
 
 QSODataDialog::~QSODataDialog() {
 }
 
+void
+QSODataDialog::OnFieldChanged(wxCommandEvent& event) {
+	// If there's no record currently being added, enable
+	// the "add" button
+
+	if (_newrec < 0) {
+		_recadd_ctrl->Enable(true);
+		return;
+	}
+	_recadd_ctrl->Enable(false);
+	// If we're not on the record pending add, get out
+	if (_newrec != _recno)
+		return;
+	// If there's an error in the data, can't Add.
+	if (!wxDialog::TransferDataFromWindow())
+		return;
+	if (_call_ctrl->GetValue() == wxT(""))	// No callsign
+		return;
+	if (_date_ctrl->GetValue() == wxT(""))	// No date
+		return;
+	if (_time_ctrl->GetValue() == wxT(""))	// No time
+		return;
+	// All is OK, allow save.
+	_newrec = -1;
+	_recadd_ctrl->Enable(true);
+}
+
 bool
 QSODataDialog::TransferDataFromWindow() {
 	tqslTrace("QSODataDialog::TransferDataFromWindow");
@@ -372,58 +430,82 @@ QSODataDialog::TransferDataFromWindow() {
 	if (_band < 0 || _band >= static_cast<int>(valid_bands.size()))
 		return false;
 	rec._band = valid_bands[_band].value;
-	rec._rxband = valid_rxbands[_rxband].value;
+	if (_rxband < 0) {
+		rec._rxband = wxT("");
+	} else {
+		rec._rxband = valid_rxbands[_rxband].value;
+	}
 	rec._freq.Trim(FALSE).Trim(TRUE);
 	rec._rxfreq.Trim(FALSE).Trim(TRUE);
-	rec._propmode = valid_propmodes[_propmode].value;
-	rec._satellite = valid_satellites[_satellite].value;
+	if (_propmode < 0) {
+		rec._propmode = wxT("");
+	} else {
+		rec._propmode = valid_propmodes[_propmode].value;
+	}
+	if (_satellite < 0) {
+		rec._satellite = wxT("");
+	} else {
+		rec._satellite = valid_satellites[_satellite].value;
+	}
 
 	double freq;
+	// Set locale to "C" so the . is forced as a decimal point
+	char *oldloc = setlocale(LC_ALL, "C");
 
-	if (!rec._freq.IsEmpty()) {
-		if (!rec._freq.ToDouble(&freq)) {
-			wxMessageBox(wxT("QSO Frequency is invalid"), wxT("QSO Data Error"),
+		if (!rec._freq.IsEmpty()) {
+			if (!rec._freq.ToDouble(&freq)) {
+				wxMessageBox(_("QSO Frequency is invalid"), _("QSO Data Error"),
+					wxOK | wxICON_EXCLAMATION, this);
+				setlocale(LC_ALL, oldloc);
+				return false;
+			}
+			freq = freq * 1000.0;		// Freq is is MHz but the limits are in KHz
+			if (freq < valid_bands[_band].low || (valid_bands[_band].high > 0 && freq > valid_bands[_band].high)) {
+				wxMessageBox(_("QSO Frequency is out of range for the selected band"), _("QSO Data Error"),
+					wxOK | wxICON_EXCLAMATION, this);
+				setlocale(LC_ALL, oldloc);
+				return false;
+			}
+		}
+		if (!rec._rxfreq.IsEmpty()) {
+			if (!rec._rxfreq.ToDouble(&freq)) {
+				wxMessageBox(_("QSO RX Frequency is invalid"), _("QSO Data Error"),
+					wxOK | wxICON_EXCLAMATION, this);
+				setlocale(LC_ALL, oldloc);
+				return false;
+			}
+			freq = freq * 1000.0;		// Freq is is MHz but the limits are in KHz
+			if (freq < valid_rxbands[_rxband].low || (valid_rxbands[_rxband].high > 0 && freq > valid_rxbands[_rxband].high)) {
+				wxMessageBox(_("QSO RX Frequency is out of range for the selected band"), _("QSO Data Error"),
+					wxOK | wxICON_EXCLAMATION, this);
+				setlocale(LC_ALL, oldloc);
+				return false;
+			}
+		}
+		// No other numeric conversions, revert to original locale
+		setlocale(LC_ALL, oldloc);
+		if (rec._freq.IsEmpty() && rec._band.IsEmpty()) {
+			wxMessageBox(_("You must select a band or enter a frequency"), _("QSO Data Error"),
 				wxOK | wxICON_EXCLAMATION, this);
 			return false;
 		}
-		freq = freq * 1000.0;		// Freq is is MHz but the limits are in KHz
-		if (freq < valid_bands[_band].low || (valid_bands[_band].high > 0 && freq > valid_bands[_band].high)) {
-			wxMessageBox(wxT("QSO Frequency is out of range for the selected band"), wxT("QSO Data Error"),
+		if (!_isend && rec._call == wxT("")) {
+			wxMessageBox(_("Call Sign cannot be empty"), _("QSO Data Error"),
 				wxOK | wxICON_EXCLAMATION, this);
 			return false;
 		}
-	}
-
-	if (!rec._rxfreq.IsEmpty()) {
-		if (!rec._rxfreq.ToDouble(&freq)) {
-			wxMessageBox(wxT("QSO RX Frequency is invalid"), wxT("QSO Data Error"),
+		if (rec._propmode == wxT("SAT") && rec._satellite == wxT("")) {
+			wxMessageBox(_("'Satellite' propagation mode selected, so a Satellite must be chosen"), _("QSO Data Error"),
 				wxOK | wxICON_EXCLAMATION, this);
 			return false;
 		}
-		freq = freq * 1000.0;		// Freq is is MHz but the limits are in KHz
-		if (freq < valid_rxbands[_rxband].low || (valid_rxbands[_rxband].high > 0 && freq > valid_rxbands[_rxband].high)) {
-			wxMessageBox(wxT("QSO RX Frequency is out of range for the selected band"), wxT("QSO Data Error"),
+		if (rec._propmode != wxT("SAT") && rec._satellite != wxT("")) {
+			wxMessageBox(_("Satellite choice requires that Propagation Mode be 'Satellite'"), _("QSO Data Error"),
 				wxOK | wxICON_EXCLAMATION, this);
 			return false;
 		}
-	}
-	if (!_isend && rec._call == wxT("")) {
-		wxMessageBox(wxT("Call Sign cannot be empty"), wxT("QSO Data Error"),
-			wxOK | wxICON_EXCLAMATION, this);
-		return false;
-	}
-	if (rec._propmode == wxT("SAT") && rec._satellite == wxT("")) {
-		wxMessageBox(wxT("'Satellite' propagation mode selected, so\na a Satellite must be chosen"), wxT("QSO Data Error"),
-			wxOK | wxICON_EXCLAMATION, this);
-		return false;
-	}
-	if (rec._propmode != wxT("SAT") && rec._satellite != wxT("")) {
-		wxMessageBox(wxT("Satellite choice requires that\nPropagation Mode be 'Satellite'"), wxT("QSO Data Error"),
-			wxOK | wxICON_EXCLAMATION, this);
-		return false;
-	}
 	if (_reclist != 0)
-			(*_reclist)[_recno-1] = rec;
+		(*_reclist)[_recno-1] = rec;
 	return true;
 }
 
@@ -434,7 +516,7 @@ QSODataDialog::TransferDataToWindow() {
 	if ((it = find(valid_modes.begin(), valid_modes.end(), rec._mode.Upper())) != valid_modes.end())
 		_mode = distance(valid_modes.begin(), it);
 	else
-		wxLogWarning(wxT("QSO Data: Invalid Mode ignored - %s"), (const char*) rec._mode.Upper().ToUTF8());
+		wxLogWarning(_("QSO Data: Invalid Mode ignored - %s"), (const char*) rec._mode.Upper().ToUTF8());
 	if ((it = find(valid_bands.begin(), valid_bands.end(), rec._band.Upper())) != valid_bands.end())
 		_band = distance(valid_bands.begin(), it);
 	if ((it = find(valid_rxbands.begin(), valid_rxbands.end(), rec._rxband.Upper())) != valid_rxbands.end())
@@ -446,17 +528,103 @@ QSODataDialog::TransferDataToWindow() {
 	return wxDialog::TransferDataToWindow();
 }
 
+bool
+QSODataDialog::WriteQSOFile(QSORecordList& recs, const char *fname) {
+	tqslTrace("QSODataDialog::writeQSOFile", "fname=%s", fname);
+	if (recs.empty()) {
+		wxLogWarning(_("No QSO records"));
+		return true;
+	}
+	wxString s_fname;
+	if (fname)
+		s_fname = wxString::FromUTF8(fname);
+	wxString path, basename, type;
+	wxFileName::SplitPath(s_fname, &path, &basename, &type);
+	if (type != wxT(""))
+		basename += wxT(".") + type;
+	else
+		basename += wxT(".adi");
+	if (path == wxT(""))
+		path = wxConfig::Get()->Read(wxT("QSODataPath"), wxT(""));
+	s_fname = wxFileSelector(_("Save File"), path, basename, wxT("adi"),
+#if !defined(__APPLE__) && !defined(_WIN32)
+		_("ADIF files (*.adi;*.adif;*.ADI;*.ADIF)|*.adi;*.adif;*.ADI;*.ADIF|All files (*.*)|*.*"),
+#else
+		_("ADIF files (*.adi;*.adif)|*.adi;*.adif|All files (*.*)|*.*"),
+#endif
+		wxFD_SAVE|wxFD_OVERWRITE_PROMPT, this);
+	if (s_fname == wxT("")) { // Cancel
+		return false;
+	}
+	wxConfig::Get()->Write(wxT("QSODataPath"), wxPathOnly(s_fname));
+
+#ifdef _WIN32
+	wchar_t* lfn = utf8_to_wchar(s_fname.ToUTF8());
+	ofstream out(lfn, ios::out|ios::trunc|ios::binary);
+	free_wchar(lfn);
+#else
+	ofstream out(s_fname.ToUTF8(), ios::out|ios::trunc|ios::binary);
+#endif
+	if (!out.is_open())
+		return false;
+	unsigned char buf[256];
+	QSORecordList::iterator it;
+	for (it = recs.begin(); it != recs.end(); it++) {
+		wxString dtstr;
+		tqsl_adifMakeField("CALL", 0, (const unsigned char*)(const char *)it->_call.ToUTF8(), -1, buf, sizeof buf);
+		out << buf << endl;
+		tqsl_adifMakeField("BAND", 0, (const unsigned char*)(const char *)it->_band.ToUTF8(), -1, buf, sizeof buf);
+		out << "   " << buf << endl;
+		tqsl_adifMakeField("MODE", 0, (const unsigned char*)(const char *)it->_mode.ToUTF8(), -1, buf, sizeof buf);
+		out << "   " << buf << endl;
+		dtstr.Printf(wxT("%04d%02d%02d"), it->_date.year, it->_date.month, it->_date.day);
+		tqsl_adifMakeField("QSO_DATE", 0, (const unsigned char*)(const char *)dtstr.ToUTF8(), -1, buf, sizeof buf);
+		out << "   " << buf << endl;
+		dtstr.Printf(wxT("%02d%02d%02d"), it->_time.hour, it->_time.minute, it->_time.second);
+		tqsl_adifMakeField("TIME_ON", 0, (const unsigned char*)(const char *)dtstr.ToUTF8(), -1, buf, sizeof buf);
+		out << "   " << buf << endl;
+		if (it->_freq != wxT("")) {
+			tqsl_adifMakeField("FREQ", 0, (const unsigned char*)(const char *)it->_freq.ToUTF8(), -1, buf, sizeof buf);
+			out << "   " << buf << endl;
+		}
+		if (it->_rxband != wxT("")) {
+			tqsl_adifMakeField("BAND_RX", 0, (const unsigned char*)(const char *)it->_rxband.ToUTF8(), -1, buf, sizeof buf);
+			out << "   " << buf << endl;
+		}
+		if (it->_rxfreq != wxT("")) {
+			tqsl_adifMakeField("FREQ_RX", 0, (const unsigned char*)(const char *)it->_rxfreq.ToUTF8(), -1, buf, sizeof buf);
+			out << "   " << buf << endl;
+		}
+		if (it->_propmode != wxT("")) {
+			tqsl_adifMakeField("PROP_MODE", 0, (const unsigned char*)(const char *)it->_propmode.ToUTF8(), -1, buf, sizeof buf);
+			out << "   " << buf << endl;
+		}
+		if (it->_satellite != wxT("")) {
+			tqsl_adifMakeField("SAT_NAME", 0, (const unsigned char*)(const char *)it->_satellite.ToUTF8(), -1, buf, sizeof buf);
+			out << "   " << buf << endl;
+		}
+		out << "<EOR>" << endl;
+	}
+	out.close();
+	wxLogMessage(_("Wrote %d QSO records to %s"), static_cast<int>(recs.size()), s_fname.c_str());
+	return true;
+}
+
 void
 QSODataDialog::OnOk(wxCommandEvent&) {
 	tqslTrace("QSODataDialog::OnOk");
+	if (!Validate())
+		return;
 	_isend = true;
 	TransferDataFromWindow();
 	_isend = false;
 	if (rec._call == wxT("") && _recno == static_cast<int>(_reclist->size())) {
 		_reclist->erase(_reclist->begin() + _recno - 1);
-		EndModal(wxID_OK);
+		if (WriteQSOFile(*_reclist, _filename.ToUTF8()))
+			EndModal(wxID_OK);
 	} else if (Validate() && TransferDataFromWindow()) {
-		EndModal(wxID_OK);
+		if (WriteQSOFile(*_reclist, _filename.ToUTF8()))
+			EndModal(wxID_OK);
 	}
 }
 
@@ -478,10 +646,10 @@ QSODataDialog::SetRecno(int new_recno) {
 	tqslTrace("QSODataDialog::SetRecno", "new_recno=%d", new_recno);
 	if (_reclist == NULL || new_recno < 1)
 		return;
-	if (TransferDataFromWindow()) {
+	if (Validate() && TransferDataFromWindow()) {
 //   		(*_reclist)[_recno-1] = rec;
 		if (_reclist && new_recno > static_cast<int>(_reclist->size())) {
-			new_recno = _reclist->size() + 1;
+			_newrec = _reclist->size() + 1;
 			QSORecord newrec;
 			// Copy QSO fields from current record
 			if (_recno > 0) {
@@ -501,7 +669,24 @@ QSODataDialog::SetRecno(int new_recno) {
 void
 QSODataDialog::OnRecDown(wxCommandEvent&) {
 	tqslTrace("QSODataDialog::OnRecDown");
-	SetRecno(_recno - 1);
+	if (_reclist == 0)
+		return;
+	if (_recno == _newrec) { 		// Backing up from a record being added
+		if (rec._call == wxT("")) { 	// And the call is empty
+			_reclist->erase(_reclist->begin() + _recno - 1);
+			if (_reclist->empty())
+				_reclist->push_back(QSORecord());
+			if (_recno > static_cast<int>(_reclist->size()))
+				_recno = _reclist->size();
+			rec = (*_reclist)[_recno-1];
+			TransferDataToWindow();
+			SetRecno(_recno);
+		}
+		_newrec = -1;
+		_recadd_ctrl->Enable(true);
+	} else {
+		SetRecno(_recno - 1);
+	}
 }
 
 void
@@ -513,6 +698,21 @@ QSODataDialog::OnRecUp(wxCommandEvent&) {
 void
 QSODataDialog::OnRecBottom(wxCommandEvent&) {
 	tqslTrace("QSODataDialog::OnRecBottom");
+	if (_reclist == 0)
+		return;
+	if (_recno == _newrec) { 		// Backing up from a record being added
+		if (rec._call == wxT("")) { 	// And the call is empty
+			_reclist->erase(_reclist->begin() + _recno - 1);
+			if (_reclist->empty())
+				_reclist->push_back(QSORecord());
+			if (_recno > static_cast<int>(_reclist->size()))
+				_recno = _reclist->size();
+			rec = (*_reclist)[_recno-1];
+			TransferDataToWindow();
+		}
+		_newrec = -1;
+		_recadd_ctrl->Enable(true);
+	}
 	SetRecno(1);
 }
 
@@ -521,6 +721,19 @@ QSODataDialog::OnRecTop(wxCommandEvent&) {
 	tqslTrace("QSODataDialog::OnRecTop");
 	if (_reclist == 0)
 		return;
+	if (_recno == _newrec) { 		// Backing up from a record being added
+		if (rec._call == wxT("")) { 	// And the call is empty
+			_reclist->erase(_reclist->begin() + _recno - 1);
+			if (_reclist->empty())
+				_reclist->push_back(QSORecord());
+			if (_recno > static_cast<int>(_reclist->size()))
+				_recno = _reclist->size();
+			rec = (*_reclist)[_recno-1];
+			TransferDataToWindow();
+		}
+		_newrec = -1;
+		_recadd_ctrl->Enable(true);
+	}
 	SetRecno(_reclist->size());
 }
 
@@ -559,4 +772,5 @@ QSODataDialog::UpdateControls() {
 	_recno_ctrl->SetValue(wxString::Format(wxT("%d"), _recno));
 	_recno_label_ctrl->SetLabel(wxString::Format(wxT("%d QSO Record%hs"), static_cast<int>(_reclist->size()),
 		(_reclist->size() == 1) ? "" : "s"));
+	_recadd_ctrl->Enable(_newrec < 0);
 }
