@@ -41,7 +41,13 @@ using std::vector;
 
 class QSORecord {
  public:
-	QSORecord() { _mode = wxT("CW"); _band = wxT("160M"); _rxband = wxT(""); }
+	QSORecord() {
+		_call = _freq = _rxfreq = _rxband = _propmode = _satellite = wxT("");
+		_mode = wxT("CW");
+		_band = wxT("160M");
+		tqsl_initDate(&_date, NULL);
+		tqsl_initTime(&_time, NULL);
+	}
 
 	wxString _call, _freq, _rxfreq;
 	wxString _mode, _band, _rxband;
@@ -54,12 +60,14 @@ typedef vector<QSORecord> QSORecordList;
 
 class QSODataDialog : public wxDialog  {
  public:
-	QSODataDialog(wxWindow *parent, wxHtmlHelpController *help, QSORecordList *reclist = 0,
-		wxWindowID id = -1, const wxString& title = wxT("QSO Data"));
+	QSODataDialog(wxWindow *parent, wxString &filename, wxHtmlHelpController *help, QSORecordList *reclist = 0,
+		wxWindowID id = -1, const wxString& title = _("QSO Data"));
 	~QSODataDialog();
 	wxString GetMode() const;
 	bool SetMode(const wxString&);
+	bool WriteQSOFile(QSORecordList& recs, const char *fname);
 	QSORecord rec;
+
 	virtual bool TransferDataToWindow();
 	virtual bool TransferDataFromWindow();
 
@@ -73,19 +81,25 @@ class QSODataDialog : public wxDialog  {
 	void OnRecTop(wxCommandEvent&);
 	void OnRecNew(wxCommandEvent&);
 	void OnRecDelete(wxCommandEvent&);
+	void OnFieldChanged(wxCommandEvent&);
 	void SetRecno(int recno);
 	void UpdateControls();
 
 	wxTextCtrl *_recno_ctrl;
 	wxTextCtrl *_call_ctrl;
+	wxTextCtrl *_date_ctrl;
+	wxTextCtrl *_time_ctrl;
 	wxStaticText *_recno_label_ctrl;
 	wxBitmapButton *_recdown_ctrl, *_recup_ctrl, *_recbottom_ctrl, *_rectop_ctrl;
+	wxButton *_recadd_ctrl;
 	QSORecordList *_reclist;
 	int _mode, _band, _rxband, _propmode, _satellite;
 	int _recno;
 	DECLARE_EVENT_TABLE()
 	bool _isend;
+	wxString _filename;
 	wxHtmlHelpController *_help;
+	int _newrec;
 };
 
 #endif
