@@ -8,26 +8,23 @@
     revision             : $Id: stationdial.cpp,v 1.6 2013/03/01 13:00:59 k1mu Exp $
  ***************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include "sysconfig.h"
-#endif
-
 #define TQSL_ID_LOW 6000
 
 #include "stationdial.h"
+#include <wx/listctrl.h>
+#include <algorithm>
+#include <iostream>
+#include <map>
+#ifdef HAVE_CONFIG_H
+#include "sysconfig.h"
+#endif
 #include "tqslwiz.h"
 #include "tqslexcept.h"
 #include "tqsltrace.h"
-#include <wx/listctrl.h>
-
-#include <algorithm>
-
 #include "tqsllib.h"
 #include "wxutil.h"
 
-#include <iostream>
-
-using namespace std;
+using std::map;
 
 #define GS_NAMELIST TQSL_ID_LOW
 #define GS_OKBUT TQSL_ID_LOW+1
@@ -40,7 +37,7 @@ using namespace std;
 #define GS_HELPBUT TQSL_ID_LOW+8
 
 class TQSLStationListBox : public wxListBox {
-public:
+ public:
 	TQSLStationListBox(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition,
 		const wxSize& size = wxDefaultSize, int n = 0, const wxString choices[] = NULL,
 		long style = 0) : wxListBox(parent, id, pos, size, n, choices, style) {}
@@ -56,27 +53,27 @@ END_EVENT_TABLE()
 void
 TQSLStationListBox::OnRightDown(wxMouseEvent& event) {
 	wxMenu menu;
-	menu.Append(GS_CMD_PROPERTIES, wxT("&Properties"));
+	menu.Append(GS_CMD_PROPERTIES, _("&Properties"));
 	PopupMenu(&menu, event.GetPosition());
 }
 
 class PropList : public wxDialog {
-public:
-	PropList(wxWindow *parent);
+ public:
+	explicit PropList(wxWindow *parent);
 	wxListCtrl *list;
 };
 
-PropList::PropList(wxWindow *parent) : wxDialog(parent, -1, wxT("Properties"), wxDefaultPosition,
+PropList::PropList(wxWindow *parent) : wxDialog(parent, -1, _("Properties"), wxDefaultPosition,
 	wxSize(400, 300)) {
-	tqslTrace("PropList::PropList", "parent=0x%lx", (void *)parent);
+	tqslTrace("PropList::PropList", "parent=0x%lx", reinterpret_cast<void *>(parent));
 	list = new wxListCtrl(this, -1, wxDefaultPosition, wxDefaultSize, wxLC_REPORT|wxLC_SINGLE_SEL);
-	list->InsertColumn(0, wxT("Name"), wxLIST_FORMAT_LEFT, 100);
-	list->InsertColumn(1, wxT("Value"), wxLIST_FORMAT_LEFT, 300);
+	list->InsertColumn(0, _("Name"), wxLIST_FORMAT_LEFT, 100);
+	list->InsertColumn(1, _("Value"), wxLIST_FORMAT_LEFT, 300);
 	wxLayoutConstraints *c = new wxLayoutConstraints;
-	c->top.SameAs            (this, wxTop);
-	c->left.SameAs        (this, wxLeft);
-	c->right.SameAs        (this, wxRight);
-	c->height.PercentOf    (this, wxHeight, 66);
+	c->top.SameAs(this, wxTop);
+	c->left.SameAs(this, wxLeft);
+	c->right.SameAs(this, wxRight);
+	c->height.PercentOf(this, wxHeight, 66);
 	list->SetConstraints(c);
 	CenterOnParent();
 }
@@ -85,7 +82,8 @@ static void
 check_tqsl_error(int rval) {
 	if (rval == 0)
 		return;
-	throw TQSLException(tqsl_getErrorString());
+	wxString serr = getLocalizedErrorString();
+        wxLogError(serr);
 }
 
 static bool
@@ -117,22 +115,22 @@ TQSLGetStationNameDialog::OnSetFocus(wxFocusEvent& event) {
 		if (issave) {
 			if (name_entry)
 				name_entry->SetFocus();
-		} else if (namelist)
+		} else if (namelist) {
 			namelist->SetFocus();
+		}
 	}
 }
 
 void
 TQSLGetStationNameDialog::OnOk(wxCommandEvent&) {
-	tqslTrace("TQSLGetStationNameDialog::OnOk");
 	wxString s = name_entry->GetValue().Trim().Trim(false);
-	if (editonly)
+	tqslTrace("TQSLGetStationNameDialog::OnOk", "selected = %s", S(s));
+	if (editonly) {
 		EndModal(wxID_CANCEL);
-	else if (s != wxT("")) {
+	} else if (s != wxT("")) {
 		_selected = s;
 		EndModal(wxID_OK);
 	}
-
 }
 
 void
@@ -155,23 +153,23 @@ TQSLGetStationNameDialog::RefreshList() {
 		item it;
 		char buf[256];
 		check_tqsl_error(tqsl_getStationLocationName(loc, i, buf, sizeof buf));
-		it.name = wxString(buf, wxConvLocal);
+		it.name = wxString::FromUTF8(buf);
 		char cbuf[256];
 		check_tqsl_error(tqsl_getStationLocationCallSign(loc, i, cbuf, sizeof cbuf));
-		it.call = wxString(cbuf, wxConvLocal);
+		it.call = wxString::FromUTF8(cbuf);
 		it.label = it.call + wxT(" - ") + it.name;
 		item_data.push_back(it);
 	}
 	sort(item_data.begin(), item_data.end(), itemLess);
-	for (int i = 0; i < (int)item_data.size(); i++)
+	for (int i = 0; i < static_cast<int>(item_data.size()); i++)
 		namelist->Append(item_data[i].label, &(item_data[i].name));
 }
 
 TQSLGetStationNameDialog::TQSLGetStationNameDialog(wxWindow *parent, wxHtmlHelpController *help, const wxPoint& pos,
 	bool i_issave, const wxString& title, const wxString& okLabel, bool i_editonly)
-	: wxDialog(parent, -1, wxT("Select Station Data"), pos), issave(i_issave), editonly(i_editonly),
+	: wxDialog(parent, -1, _("Select Station Data"), pos), issave(i_issave), editonly(i_editonly),
 	newbut(0), modbut(0), updating(false), firstFocused(false), _help(help) {
-	tqslTrace("TQSLGetStationNameDialog::TQSLGetStationNameDialog", "parent=0x%lx, i_issave=%d, title=%s, okLabel=%s, i_editonly=%d", parent, i_issave, _S(title), _S(okLabel), i_editonly);
+	tqslTrace("TQSLGetStationNameDialog::TQSLGetStationNameDialog", "parent=0x%lx, i_issave=%d, title=%s, okLabel=%s, i_editonly=%d", parent, i_issave, S(title), S(okLabel), i_editonly);
 	wxBoxSizer *topsizer = new wxBoxSizer(wxHORIZONTAL);
 	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
 	wxSize text_size = getTextSize(this);
@@ -180,15 +178,16 @@ TQSLGetStationNameDialog::TQSLGetStationNameDialog(wxWindow *parent, wxHtmlHelpC
 	if (title != wxT(""))
 		SetTitle(title);
 	else if (issave)
-		SetTitle(wxT("Save Station Data"));
+		SetTitle(_("Save Station Data"));
 	// List
 	namelist = new TQSLStationListBox(this, GS_NAMELIST, wxDefaultPosition,
 		wxSize(control_width, text_size.GetHeight()*10),
 		0, 0, wxLB_MULTIPLE|wxLB_HSCROLL|wxLB_ALWAYS_SB);
 	sizer->Add(namelist, 1, wxALL|wxEXPAND, 10);
 	RefreshList();
-	sizer->Add(new wxStaticText(this, -1, issave ? wxT("Enter a name for this Station Location") :
-		wxT("Selected Station Location")), 0, wxLEFT|wxRIGHT|wxTOP|wxEXPAND, 10);
+	sizer->Add(new wxStaticText(this, -1,
+		issave ? _("Enter a name for this Station Location") : _("Selected Station Location")),
+		0, wxLEFT|wxRIGHT|wxTOP|wxEXPAND, 10);
 	name_entry = new wxTextCtrl(this, GS_NAMEENTRY, wxT(""), wxDefaultPosition,
 		wxSize(control_width, -1));
 	if (!issave)
@@ -197,22 +196,22 @@ TQSLGetStationNameDialog::TQSLGetStationNameDialog(wxWindow *parent, wxHtmlHelpC
 	topsizer->Add(sizer, 1, 0, 0);
 	wxBoxSizer *button_sizer = new wxBoxSizer(wxVERTICAL);
 	if (!issave) {
-		newbut = new wxButton(this, GS_NEWBUT, wxT("New..."));
+		newbut = new wxButton(this, GS_NEWBUT, _("New..."));
 		newbut->Enable(TRUE);
 		button_sizer->Add(newbut, 0, wxALL|wxALIGN_TOP, 3);
-		modbut = new wxButton(this, GS_MODIFYBUT, wxT("Edit..."));
+		modbut = new wxButton(this, GS_MODIFYBUT, _("Edit..."));
 		modbut->Enable(FALSE);
 		button_sizer->Add(modbut, 0, wxALL|wxALIGN_TOP, 3);
 	}
-	delbut = new wxButton(this, GS_DELETEBUT, wxT("Delete"));
+	delbut = new wxButton(this, GS_DELETEBUT, _("Delete"));
 	delbut->Enable(FALSE);
 	button_sizer->Add(delbut, 0, wxALL|wxALIGN_TOP, 3);
 	button_sizer->Add(new wxStaticText(this, -1, wxT("")), 1, wxEXPAND);
 	if (_help)
-		button_sizer->Add(new wxButton(this, GS_HELPBUT, wxT("Help") ), 0, wxALL|wxALIGN_BOTTOM, 3);
+		button_sizer->Add(new wxButton(this, GS_HELPBUT, _("Help") ), 0, wxALL|wxALIGN_BOTTOM, 3);
 	if (!editonly)
-		button_sizer->Add(new wxButton(this, GS_CANCELBUT, wxT("Cancel") ), 0, wxALL|wxALIGN_BOTTOM, 3);
-	okbut = new wxButton(this, GS_OKBUT, okLabel );
+		button_sizer->Add(new wxButton(this, GS_CANCELBUT, _("Cancel") ), 0, wxALL|wxALIGN_BOTTOM, 3);
+	okbut = new wxButton(this, GS_OKBUT, okLabel);
 	button_sizer->Add(okbut, 0, wxALL|wxALIGN_BOTTOM, 3);
 	topsizer->Add(button_sizer, 0, wxTOP|wxBOTTOM|wxRIGHT|wxEXPAND, 7);
 	hack = (namelist->GetCount() > 0) ? true : false;
@@ -249,7 +248,7 @@ TQSLGetStationNameDialog::UpdateControls() {
 	wxArrayInt newsels;
 	namelist->GetSelections(newsels);
 	int newsel = -1;
-	for (int i = 0; i < (int)newsels.GetCount(); i++) {
+	for (int i = 0; i < static_cast<int>(newsels.GetCount()); i++) {
 		if (sels.Index(newsels[i]) == wxNOT_FOUND) {
 			newsel = newsels[i];
 			break;
@@ -257,7 +256,7 @@ TQSLGetStationNameDialog::UpdateControls() {
 	}
 //cout << "newsel: " << newsel << endl;
 	if (newsel > -1) {
-		for (int i = 0; i < (int)newsels.GetCount(); i++) {
+		for (int i = 0; i < static_cast<int>(newsels.GetCount()); i++) {
 			if (newsels[i] != newsel)
 				namelist->Deselect(newsels[i]);
 		}
@@ -266,7 +265,7 @@ TQSLGetStationNameDialog::UpdateControls() {
 	int idx = (sels.GetCount() > 0) ? sels[0] : -1;
 //cout << "UpdateControls selection: " << idx << endl;
 	if (idx >= 0)
-		name_entry->SetValue((idx < 0) ? wxT("") : *(wxString *)namelist->GetClientData(idx));
+		name_entry->SetValue((idx < 0) ? wxT("") : *reinterpret_cast<wxString *>(namelist->GetClientData(idx)));
 	UpdateButtons();
 //cout << "UpdateControls selection(1): " << idx << endl;
 	updating = false;
@@ -280,11 +279,11 @@ TQSLGetStationNameDialog::OnDelete(wxCommandEvent&) {
 	int idx = (newsels.GetCount() > 0) ? newsels[0] : -1;
 	if (idx < 0)
 		return;
-	wxString name = *(wxString *)namelist->GetClientData(idx);
+	wxString name = *reinterpret_cast<wxString *>(namelist->GetClientData(idx));
 	if (name == wxT(""))
 		return;
-	if (wxMessageBox(wxString(wxT("Delete \"")) + name + wxT("\"?"), wxT("TQSL Confirm"), wxYES_NO|wxCENTRE, this) == wxYES) {
-		check_tqsl_error(tqsl_deleteStationLocation(name.mb_str()));
+	if (wxMessageBox(wxString(_("Delete \"")) + name + wxT("\"?"), _("TQSL Confirm"), wxYES_NO|wxCENTRE, this) == wxYES) {
+		check_tqsl_error(tqsl_deleteStationLocation(name.ToUTF8()));
 		if (!issave)
 			name_entry->Clear();
 		RefreshList();
@@ -311,9 +310,13 @@ cout << "OnNamelist hack" << endl;
 void
 TQSLGetStationNameDialog::OnDblClick(wxCommandEvent& event) {
 	tqslTrace("TQSLGetStationNameDialog::OnDblClick");
-	if(editonly) { OnNamelist(event); EndModal(wxID_MORE); }
-	else { UpdateControls(); OnOk(event); } //updatecontrols sets up the text field that ok checks
-	
+	if(editonly) {
+		OnNamelist(event);
+		EndModal(wxID_MORE);
+	} else {
+		UpdateControls();
+		OnOk(event);
+	} //updatecontrols sets up the text field that ok checks
 }
 
 void
@@ -341,6 +344,25 @@ TQSLGetStationNameDialog::OnNameChange(wxCommandEvent&) {
 	UpdateButtons();
 }
 
+// Location fields, here for translation purposes
+#ifdef tqsltranslate
+static const char* labels[] = {
+	__("State"),
+	__("Call Sign"),
+	__("Province"),
+	__("Continent"),
+	__("CQ Zone"),
+	__("DXCC Entity"),
+	__("Grid Square"),
+	__("IOTA ID"),
+	__("ITU Zone"),
+	__("Oblast"),
+	__("County"),
+	__("State"),
+	__("WPX Prefix")
+}
+#endif
+
 void
 TQSLGetStationNameDialog::DisplayProperties(wxCommandEvent&) {
 	tqslTrace("TQSLGetStationNameDialog::DisplayProperties");
@@ -349,39 +371,40 @@ TQSLGetStationNameDialog::DisplayProperties(wxCommandEvent&) {
 	int idx = (newsels.GetCount() > 0) ? newsels[0] : -1;
 	if (idx < 0)
 		return;
-	wxString name = *(wxString *)namelist->GetClientData(idx);
+	wxString name = *reinterpret_cast<wxString *>(namelist->GetClientData(idx));
 	if (name == wxT(""))
 		return;
 	tQSL_Location loc;
 	try {
-		map<wxString,wxString> props;
-		check_tqsl_error(tqsl_getStationLocation(&loc, name.mb_str()));
+		map<wxString, wxString> props;
+		check_tqsl_error(tqsl_getStationLocation(&loc, name.ToUTF8()));
 		do {
 			int nfield;
 			check_tqsl_error(tqsl_getNumLocationField(loc, &nfield));
 			for (int i = 0; i < nfield; i++) {
 				char buf[256];
 				check_tqsl_error(tqsl_getLocationFieldDataLabel(loc, i, buf, sizeof buf));
-				wxString key = wxString(buf, wxConvLocal);
+				wxString key = wxGetTranslation(wxString::FromUTF8(buf));
 				int type;
 				check_tqsl_error(tqsl_getLocationFieldDataType(loc, i, &type));
 				if (type == TQSL_LOCATION_FIELD_DDLIST || type == TQSL_LOCATION_FIELD_LIST) {
 					int sel;
 					check_tqsl_error(tqsl_getLocationFieldIndex(loc, i, &sel));
 					check_tqsl_error(tqsl_getLocationFieldListItem(loc, i, sel, buf, sizeof buf));
-				} else
+				} else {
 					check_tqsl_error(tqsl_getLocationFieldCharData(loc, i, buf, sizeof buf));
-				props[key] = wxString(buf, wxConvLocal);
+				}
+				props[key] = wxString::FromUTF8(buf);
 			}
 			int rval;
 			if (tqsl_hasNextStationLocationCapture(loc, &rval) || !rval)
 				break;
 			check_tqsl_error(tqsl_nextStationLocationCapture(loc));
-		} while(1);
+		} while (1);
 		check_tqsl_error(tqsl_endStationLocationCapture(&loc));
 		PropList plist(this);
 		int i = 0;
-		for (map<wxString,wxString>::iterator it = props.begin(); it != props.end(); it++) {
+		for (map<wxString, wxString>::iterator it = props.begin(); it != props.end(); it++) {
 			plist.list->InsertItem(i, it->first);
 			plist.list->SetItem(i, 1, it->second);
 			i++;
@@ -389,20 +412,20 @@ TQSLGetStationNameDialog::DisplayProperties(wxCommandEvent&) {
 		}
 		plist.ShowModal();
 	}
-	catch (TQSLException& x) {
+	catch(TQSLException& x) {
 		wxLogError(wxT("%hs"), x.what());
 	}
 }
 
 void
 TQSLGetStationNameDialog::SelectName(const wxString& name) {
-	tqslTrace("TQSLGetStationNameDialog::SelectName", "name=%s", _S(name));
+	tqslTrace("TQSLGetStationNameDialog::SelectName", "name=%s", S(name));
 	wxArrayInt sels;
 	namelist->GetSelections(sels);
-	for (int i = 0; i < (int)sels.GetCount(); i++)
+	for (int i = 0; i < static_cast<int>(sels.GetCount()); i++)
 		namelist->Deselect(sels[i]);
-	for (int i = 0; i < (int) namelist->GetCount(); i++) {
-		if (name == *(wxString *)namelist->GetClientData(i)) {
+	for (int i = 0; i < static_cast<int>(namelist->GetCount()); i++) {
+		if (name == *reinterpret_cast<wxString *>(namelist->GetClientData(i))) {
 			namelist->SetSelection(i, TRUE);
 			break;
 		}
@@ -414,10 +437,13 @@ TQSLGetStationNameDialog::SelectName(const wxString& name) {
 int
 TQSLGetStationNameDialog::ShowModal() {
 	tqslTrace("TQSLGetStationNameDialog::ShowModal");
-	if (namelist->GetCount() == 0 && !issave)
-		wxMessageBox(wxT("You have no Station Locations defined.\n\n")
-			wxT("You must define at least one Station Location to use for signing.\n")
-			wxT("Use the \"New\" Button of the dialog you're about to see to\ndefine a Station Location."),
-			wxT("TQSL Warning"), wxOK, this);
+	if (namelist->GetCount() == 0 && !issave) {
+		wxString msg = _("You have no Station Locations defined.");
+			msg += wxT("\n\n");
+			msg += _("You must define at least one Station Location to use for signing.");
+			msg += wxT("\n");
+			msg += _("Use the \"New\" Button of the dialog you're about to see to define a Station Location.");
+		wxMessageBox(msg, _("TQSL Warning"), wxOK, this);
+	}
 	return wxDialog::ShowModal();
 }
