@@ -10,10 +10,8 @@
 #ifdef HAVE_CONFIG_H
 #include "sysconfig.h"
 #endif
+#include "tqsllib.h"
 
-extern FILE *diagFile;
-
-void tqslTrace(const char *name, const char *format = NULL, ...);
 #define S(x) ((const char *) x.ToUTF8())
 
 #endif	// __tqslltrace_h
