@@ -29,7 +29,10 @@ END_EVENT_TABLE()
 BEGIN_EVENT_TABLE(TQSLWizCertPage, TQSLWizPage)
 	EVT_COMBOBOX(-1, TQSLWizCertPage::OnComboBoxEvent)
 	EVT_CHECKBOX(-1, TQSLWizCertPage::OnCheckBoxEvent)
+	EVT_WIZARD_PAGE_CHANGING(wxID_ANY, TQSLWizCertPage::OnPageChanging)
+#if wxMAJOR_VERSION < 3 && (wxMAJOR_VERSION != 2 && wxMINOR_VERSION != 9)
 	EVT_SIZE(TQSLWizCertPage::OnSize)
+#endif
 END_EVENT_TABLE()
 
 void
@@ -57,7 +60,6 @@ TQSLWizPage *
 TQSLWizard::GetPage(bool final) {
 	tqslTrace("TQSLWizard::GetPage", "final=%d", final);
 	int page_num;
-	page_changing = false;
 	if (final)
 		page_num = 0;
 	else if (tqsl_getStationLocationCapturePage(loc, &page_num))
@@ -79,7 +81,9 @@ TQSLWizard::GetPage(bool final) {
 }
 
 void TQSLWizCertPage::OnSize(wxSizeEvent& ev) {
-	TQSLWizPage::OnSize(ev); //fix this
+#if wxMAJOR_VERSION < 3 && (wxMAJOR_VERSION != 2 && wxMINOR_VERSION != 9)
+	TQSLWizPage::OnSize(ev);
+#endif
 	UpdateFields();
 }
 
@@ -87,14 +91,12 @@ TQSLWizPage *
 TQSLWizCertPage::GetPrev() const {
 	tqslTrace("TQSLWizCertPage::GetPrev");
 	int rval;
+
+	tqsl_setStationLocationCapturePage(loc, loc_page);
 	if (tqsl_hasPrevStationLocationCapture(loc, &rval) || !rval) {
-		GetParent()->page_changing = false;
 		return 0;
 	}
-	if (reinterpret_cast<TQSLWizard *>(GetParent())->page_changing) {
-		tqsl_prevStationLocationCapture(loc);
-		return GetParent()->GetPage();
-	}
+	tqsl_prevStationLocationCapture(loc);
 	return GetParent()->GetPage();
 }
 
@@ -103,13 +105,13 @@ TQSLWizCertPage::GetNext() const {
 	tqslTrace("TQSLWizCertPage::GetNext");
 	TQSLWizPage *newp;
 	bool final = false;
-	if (GetParent()->page_changing) {
-		int rval;
-		if (!tqsl_hasNextStationLocationCapture(loc, &rval) && rval) {
-			tqsl_nextStationLocationCapture(loc);
-		} else {
-			final = true;
-		}
+	int rval;
+
+	tqsl_setStationLocationCapturePage(loc, loc_page);
+	if (!tqsl_hasNextStationLocationCapture(loc, &rval) && rval) {
+		tqsl_nextStationLocationCapture(loc);
+	} else {
+		final = true;
 	}
 	newp = GetParent()->GetPage(final);
 	return newp;
@@ -120,31 +122,9 @@ TQSLWizCertPage::UpdateFields(int noupdate_field) {
 	tqslTrace("TQSLWizCertPage::UpdateFields", "noupdate_field=%d", noupdate_field);
 	wxSize text_size = getTextSize(this);
 
+	validate();
 	if (noupdate_field >= 0)
 		tqsl_updateStationLocationCapture(loc);
-	if (noupdate_field <= 0) {
-		int flags;
-		tqsl_getLocationFieldFlags(loc, 0, &flags);
-		if (flags == TQSL_LOCATION_FIELD_SELNXT) {
-			int index;
-			bool fwdok = false;
-			tqsl_getLocationFieldIndex(loc, 0, &index);
-			if (index > 0)
-				fwdok = true;
-			if (okEmptyCB && okEmptyCB->IsChecked())
-				fwdok = true;
-			wxWindow *but = GetParent()->FindWindow(wxID_FORWARD);
-		        if (but) {
-        			but->Enable(fwdok);
-			}
-			if (okEmptyCB) {
-				if (index > 0)
-					okEmptyCB->Enable(false);
-				else
-					okEmptyCB->Enable(true);
-			}
-		}
-	}
 	for (int i = noupdate_field+1; i < static_cast<int>(controls.size()); i++) {
 		int changed;
 		int in_type;
@@ -155,7 +135,6 @@ TQSLWizCertPage::UpdateFields(int noupdate_field) {
 			continue;
 		if (in_type == TQSL_LOCATION_FIELD_DDLIST || in_type == TQSL_LOCATION_FIELD_LIST) {
 			// Update this list
-			int text_width = 0;
 			char gabbi_name[40];
 			tqsl_getLocationFieldDataGABBI(loc, i, gabbi_name, sizeof gabbi_name);
 			int selected;
@@ -172,34 +151,26 @@ TQSLWizCertPage::UpdateFields(int noupdate_field) {
 			int nitems;
 			tqsl_getNumLocationFieldListItems(loc, i, &nitems);
 			for (int j = 0; j < nitems && j < 2000; j++) {
-				char item[80];
+				char item[200];
 				tqsl_getLocationFieldListItem(loc, i, j, item, sizeof(item));
-				/* For Alaska counties, it's pseudo-county name
-				 * followed by a '|' then the real county name.
-			 	 */
-				char *p = strchr(item, '|');
-				if (p)
-					*p = '\0';
+				// Translate the first [None] entry if it exists
+#ifdef tqsltranslate
+				__("[None]");
+#endif
 				wxString item_text = wxString::FromUTF8(item);
-				(reinterpret_cast<wxComboBox *>(controls[i]))->Append(item_text);
 				if (item_text == old_sel)
 					new_sel = j;
-				wxCoord w, h;
-				(reinterpret_cast<wxComboBox *>(controls[i]))->GetTextExtent(item_text, &w, &h);
-				if (w > text_width)
-					text_width = w;
-			}
-			if (text_width > 0) {
-				int w, h;
-				(reinterpret_cast<wxComboBox *>(controls[i]))->GetSize(&w, &h);
-				(reinterpret_cast<wxComboBox *>(controls[i]))->SetSize(text_width + text_size.GetWidth()*4, h);
+				if (j == 0)
+					item_text = wxGetTranslation(item_text);
+				(reinterpret_cast<wxComboBox *>(controls[i]))->Append(item_text);
 			}
 			if (noupdate_field < 0 && !defaulted)
 				new_sel = selected;
-			(reinterpret_cast<wxComboBox *>(controls[i]))->SetSelection(new_sel);
+			if (nitems > new_sel)
+				(reinterpret_cast<wxComboBox *>(controls[i]))->SetSelection(new_sel);
 			tqsl_setLocationFieldIndex(loc, i, new_sel);
-			if (noupdate_field >= 0)
-				tqsl_updateStationLocationCapture(loc);
+			if (nitems > new_sel)
+				(reinterpret_cast<wxComboBox *>(controls[i]))->SetSelection(new_sel);
 			(reinterpret_cast<wxComboBox *>(controls[i]))->Enable(nitems > 1);
 		} else if (in_type == TQSL_LOCATION_FIELD_TEXT) {
 			int len;
@@ -220,16 +191,15 @@ TQSLWizCertPage::UpdateFields(int noupdate_field) {
 			(reinterpret_cast<wxStaticText *>(controls[i]))->SetSize((len+1)*text_size.GetWidth(), h);
 			char buf[256];
 			tqsl_getLocationFieldCharData(loc, i, buf, sizeof buf);
-			(reinterpret_cast<wxStaticText *>(controls[i]))->SetLabel(wxString::FromUTF8(buf));
-			if (strlen(buf) == 0) {
-				this->GetParent()->FindWindow(wxID_FORWARD)->Enable(true);
-				if (valMsg)
-					(reinterpret_cast<wxStaticText *>(controls[i]))->SetLabel(wxString::FromUTF8(valMsg));
-			} else {
-				this->GetParent()->FindWindow(wxID_FORWARD)->Enable(false);
-			}
+			if (strlen(buf) > 0)
+				valMsg = wxGetTranslation(wxString::FromUTF8(buf));
+			else
+				valMsg = wxT("");
+			(reinterpret_cast<wxStaticText *>(controls[i]))->SetLabel(valMsg);
 		}
 	}
+	if (noupdate_field >= 0)
+		tqsl_updateStationLocationCapture(loc);
 }
 
 void
@@ -257,10 +227,12 @@ TQSLWizCertPage::OnCheckBoxEvent(wxCommandEvent& event) {
 TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 	: TQSLWizPage(parent, locp) {
 	tqslTrace("TQSLWizCertPage::TQSLWizCertPage", "parent=0x%lx, locp=0x%lx", reinterpret_cast<void *>(parent), reinterpret_cast<void *>(locp));
+	initialized = false;
+	errlbl = NULL;
 	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
-	int control_width = getTextSize(this).GetWidth() * 20;
+	int control_width = getTextSize(this).GetWidth() * 40;
 
-	valMsg = NULL;
+	valMsg = wxT("");
 	tqsl_getStationLocationCapturePage(loc, &loc_page);
 	wxScreenDC sdc;
 	int label_w = 0;
@@ -270,7 +242,8 @@ TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 		wxCoord w, h;
 		char label[256];
 		tqsl_getLocationFieldDataLabel(loc, i, label, sizeof label);
-		sdc.GetTextExtent(wxString::FromUTF8(label), &w, &h);
+		wxString lbl = wxGetTranslation(wxString::FromUTF8(label));
+		sdc.GetTextExtent(lbl, &w, &h);
 		if (w > label_w)
 			label_w = w;
 	}
@@ -284,10 +257,11 @@ TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 		int in_type, flags;
 		wxBoxSizer *hsizer;
 		tqsl_getLocationFieldDataLabel(loc, i, label, sizeof label);
+		wxString lbl = wxGetTranslation(wxString::FromUTF8(label));
 		tqsl_getLocationFieldInputType(loc, i, &in_type);
 		if (in_type != TQSL_LOCATION_FIELD_BADZONE) {
 			hsizer = new wxBoxSizer(wxHORIZONTAL);
-			hsizer->Add(new wxStaticText(this, -1, wxString::FromUTF8(label), wxDefaultPosition,
+			hsizer->Add(new wxStaticText(this, -1, lbl, wxDefaultPosition,
 				wxSize(label_w, -1), wxALIGN_RIGHT/*|wxST_NO_AUTORESIZE*/), 0, wxTOP, 5);
 		}
 		wxWindow *control_p = 0;
@@ -301,7 +275,7 @@ TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 					0, 0, wxCB_DROPDOWN|wxCB_READONLY);
 				if (flags & TQSL_LOCATION_FIELD_SELNXT) {
 					addCheckbox = true;
-					cbLabel = wxString::FromUTF8(label);
+					cbLabel = lbl;
 				}
 				break;
                         case TQSL_LOCATION_FIELD_TEXT:
@@ -313,8 +287,9 @@ TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 				tqsl_getLocationFieldDataLength(loc, i, &tsize);
 				tsize /= 2;
 				sdc.GetTextExtent(wxString::FromUTF8("X"), &w, &h);
-				control_p = new wxStaticText(this, -1, wxT(""), wxDefaultPosition,
+				errlbl = new wxStaticText(this, -1, wxT(""), wxDefaultPosition,
 					wxSize(w*tsize, -1), wxALIGN_LEFT|wxST_NO_AUTORESIZE);
+				control_p = errlbl;
 				break;
 		}
 		controls.push_back(control_p);
@@ -328,7 +303,7 @@ TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 
 	if (addCheckbox) {
 		wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL);
-		okEmptyCB = new wxCheckBox(this, TQSL_ID_LOW+numf, wxT("Allow 'None' for ") + cbLabel, wxDefaultPosition, wxDefaultSize);
+		okEmptyCB = new wxCheckBox(this, TQSL_ID_LOW+numf, _("Allow 'None' for ") + cbLabel, wxDefaultPosition, wxDefaultSize);
 
 		hsizer->Add(new wxStaticText(this, -1, wxT(""),
 			wxDefaultPosition, wxSize(label_w, -1), wxALIGN_RIGHT|wxST_NO_AUTORESIZE), 0, wxTOP, 5);
@@ -337,6 +312,7 @@ TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 	} else {
 		okEmptyCB = 0;
 	}
+	initialized = true;
 	UpdateFields();
 	AdjustPage(sizer, wxT("stnloc1.htm"));
 }
@@ -347,17 +323,35 @@ TQSLWizCertPage::~TQSLWizCertPage() {
 const char *
 TQSLWizCertPage::validate() {
 	tqslTrace("TQSLWizCertPage::validate");
-	static char errtxt[128];
-	wxString error;
-	error = wxT("");
+
+	if (!initialized) return 0;
+	valMsg = wxT("");
 	tqsl_setStationLocationCapturePage(loc, loc_page);
 	for (int i = 0; i < static_cast<int>(controls.size()); i++) {
 		char gabbi_name[40];
+		int in_type;
+		int flags;
 		tqsl_getLocationFieldDataGABBI(loc, i, gabbi_name, sizeof gabbi_name);
-		if (strcmp(gabbi_name, "GRIDSQUARE") == 0) {
+		tqsl_getLocationFieldInputType(loc, i, &in_type);
+		tqsl_getLocationFieldFlags(loc, i, &flags);
+		if (flags == TQSL_LOCATION_FIELD_SELNXT) {
+			int index;
+			tqsl_getLocationFieldIndex(loc, i, &index);
+			if (index <= 0 && (okEmptyCB && !okEmptyCB->IsChecked())) {
+				char label[256];
+				tqsl_getLocationFieldDataLabel(loc, i, label, sizeof label);
+				valMsg = wxString::Format(_("You must select a %hs"), label);
+			}
+			if (okEmptyCB) {
+				if (index <= 0)
+					okEmptyCB->Enable(true);
+				else
+					okEmptyCB->Enable(false);
+			}
+		} else if (strcmp(gabbi_name, "GRIDSQUARE") == 0) {
 			wxString gridVal = (reinterpret_cast<wxTextCtrl *>(controls[i]))->GetValue();
 			if (gridVal.size() == 0) {
-				return 0;
+				continue;
 			}
 			wxString editedGrids = wxT("");
 			wxStringTokenizer grids(gridVal, wxT(","));	// Comma-separated list of squares
@@ -365,29 +359,29 @@ TQSLWizCertPage::validate() {
 				wxString grid = grids.GetNextToken().Trim().Trim(false);
 				// Truncate to six character field
 				grid = grid.Left(6);
-				if (grid[0] <= 'z' && grid[1] >= 'a')
+				if (grid[0] <= 'z' && grid[0] >= 'a')
 					grid[0] = grid[0] - 'a' + 'A';	// Upper case first two
 				if (grid.size() > 1 && (grid[1] <= 'z' && grid[1] >= 'a'))
 					grid[1] = grid[1] - 'a' + 'A';
 				if (grid[0] < 'A' || grid[0] > 'R') {
-					if (error.IsEmpty())
-						error = wxString::Format(wxT("%s: Invalid Grid Square Field"), grid.c_str());
+					if (valMsg.IsEmpty())
+						valMsg = wxString::Format(_("%s: Invalid Grid Square Field"), grid.c_str());
 				}
 				if (grid.size() > 1 && (grid[1] < 'A' || grid[1] > 'R')) {
-					if (error.IsEmpty())
-						error = wxString::Format(wxT("%s: Invalid Grid Square Field"), grid.c_str());
+					if (valMsg.IsEmpty())
+						valMsg = wxString::Format(_("%s: Invalid Grid Square Field"), grid.c_str());
 				}
 				if (grid.size() > 2 && (grid[2] < '0' || grid[2] > '9')) {
-					if (error.IsEmpty())
-						error = wxString::Format(wxT("%s: Invalid Grid Square"), grid.c_str());
+					if (valMsg.IsEmpty())
+						valMsg = wxString::Format(_("%s: Invalid Grid Square"), grid.c_str());
 				}
 				if (grid.size() < 4) {
-					if (error.IsEmpty())
-						error = wxString::Format(wxT("%s: Invalid Grid Square"), grid.c_str());
+					if (valMsg.IsEmpty())
+						valMsg = wxString::Format(_("%s: Invalid Grid Square"), grid.c_str());
 				}
 				if (grid[3] < '0' || grid[3] > '9') {
-					if (error.IsEmpty())
-						error = wxString::Format(wxT("%s: Invalid Grid Square"), grid.c_str());
+					if (valMsg.IsEmpty())
+						valMsg = wxString::Format(_("%s: Invalid Grid Square"), grid.c_str());
 				}
 
 				if (grid.size() > 4 && (grid[4] <= 'Z' && grid[4] >= 'A'))
@@ -396,17 +390,17 @@ TQSLWizCertPage::validate() {
 					grid[5] = grid[5] - 'A' + 'a';
 
 				if (grid.size() > 4 && (grid[4] < 'a' || grid[4] > 'x')) {
-					if (error.IsEmpty())
-						error = wxString::Format(wxT("%s: Invalid Subsquare"), grid.c_str());
+					if (valMsg.IsEmpty())
+						valMsg = wxString::Format(_("%s: Invalid Subsquare"), grid.c_str());
 				}
 				if (grid.size() > 5 && (grid[5] < 'a' || grid[5] > 'x')) {
-					if (error.IsEmpty())
-						error = wxString::Format(wxT("%s: Invalid Subsquare"), grid.c_str());
+					if (valMsg.IsEmpty())
+						valMsg = wxString::Format(_("%s: Invalid Subsquare"), grid.c_str());
 				}
 				if (grid.size() != 6 && grid.size() != 4) {
 					// Not long enough yet or too long.
-					if (error.IsEmpty())
-						error = wxString::Format(wxT("%s: Invalid Grid Square"), grid.c_str());
+					if (valMsg.IsEmpty())
+						valMsg = wxString::Format(_("%s: Invalid Grid Square"), grid.c_str());
 				}
 				if (!editedGrids.IsEmpty())
 					editedGrids += wxT(",");
@@ -414,22 +408,31 @@ TQSLWizCertPage::validate() {
 				(reinterpret_cast<wxTextCtrl *>(controls[i]))->SetValue(editedGrids);
 				tqsl_setLocationFieldCharData(loc, i, (reinterpret_cast<wxTextCtrl *>(controls[i]))->GetValue().ToUTF8());
 			}
-			if (error.IsEmpty()) return 0;
-			strncpy(errtxt, error.ToUTF8(), sizeof errtxt);
-			return errtxt;
+		} else if (in_type == TQSL_LOCATION_FIELD_BADZONE) {
+// Possible errors, here for harvesting
+#ifdef tqsltranslate
+	static const char* verrs[] = {
+		__("Invalid zone selections for state"),
+		__("Invalid zone selections for province"),
+		__("Invalid zone selections for oblast"),
+		__("Invalid zone selections for DXCC entity");
+};
+#endif
+			char buf[256];
+			tqsl_getLocationFieldCharData(loc, i, buf, sizeof buf);
+			if (strlen(buf) > 0)
+				valMsg = wxGetTranslation(wxString::FromUTF8(buf));
+			(reinterpret_cast<wxStaticText *>(controls[i]))->SetLabel(valMsg);
 		}
 	}
+	if (errlbl) errlbl->SetLabel(valMsg);
 	return 0;
 }
 
 bool
 TQSLWizCertPage::TransferDataFromWindow() {
 	tqslTrace("TQSLWizCertPage::TransferDataFromWindow");
-	valMsg = validate();
-	if (!valMsg)
-		GetParent()->page_changing = true;
-	else
-		GetParent()->page_changing = false;
+
 	tqsl_setStationLocationCapturePage(loc, loc_page);
 	for (int i = 0; i < static_cast<int>(controls.size()); i++) {
 		int in_type;
@@ -443,13 +446,25 @@ TQSLWizCertPage::TransferDataFromWindow() {
 				break;
 		}
 	}
+	if (errlbl) errlbl->SetLabel(valMsg);
 	return true;
 }
+void
+TQSLWizCertPage::OnPageChanging(wxWizardEvent& ev) {
+	tqslTrace("TQSLWizCertPage::OnPageChanging", "Direction=", ev.GetDirection());
+
+	validate();
+	if (valMsg.Len() > 0 && ev.GetDirection()) {
+		ev.Veto();
+		wxMessageBox(valMsg, _("Error"));
+	}
+}
 
 BEGIN_EVENT_TABLE(TQSLWizFinalPage, TQSLWizPage)
 	EVT_LISTBOX(TQSL_ID_LOW, TQSLWizFinalPage::OnListbox)
 	EVT_LISTBOX_DCLICK(TQSL_ID_LOW, TQSLWizFinalPage::OnListbox)
 	EVT_TEXT(TQSL_ID_LOW+1, TQSLWizFinalPage::check_valid)
+	EVT_WIZARD_PAGE_CHANGING(wxID_ANY, TQSLWizFinalPage::OnPageChanging)
 END_EVENT_TABLE()
 
 void
@@ -465,17 +480,20 @@ TQSLWizFinalPage::OnListbox(wxCommandEvent &) {
 TQSLWizFinalPage::TQSLWizFinalPage(TQSLWizard *parent, tQSL_Location locp, TQSLWizPage *i_prev)
 	: TQSLWizPage(parent, locp), prev(i_prev) {
 	tqslTrace("TQSLWizFinalPage::TQSLWizFinalPage", "parent=0x%lx, locp=0x%lx, i_prev=0x%lx", reinterpret_cast<void *>(parent), reinterpret_cast<void *>(locp), reinterpret_cast<void *>(i_prev));
+	initialized = false;
+	errlbl = NULL;
+	valMsg = wxT("");
 	wxSize text_size = getTextSize(this);
-	int control_width = text_size.GetWidth()*30;
+	int control_width = text_size.GetWidth()*40;
 
 	int y = text_size.GetHeight();
 	sizer = new wxBoxSizer(wxVERTICAL);
 
-	wxStaticText *st = new wxStaticText(this, -1, wxT("Station Data input complete"));
+	wxStaticText *st = new wxStaticText(this, -1, _("Station Data input complete"));
 	sizer->Add(st, 0, wxALIGN_CENTER|wxTOP, 10);
 
 	// Title
-	st = new wxStaticText(this, -1, wxT("Select or enter name of this station location"));
+	st = new wxStaticText(this, -1, _("Select or enter name of this station location"));
 	sizer->Add(st, 0, wxALIGN_CENTER|wxBOTTOM, 10);
 
 	// List of existing location names
@@ -498,33 +516,52 @@ TQSLWizFinalPage::TQSLWizFinalPage(TQSLWizard *parent, tQSL_Location locp, TQSLW
 	if (namelist->GetCount() > 0)
 		namelist->SetSelection(0, FALSE);
 	// New name
-	st = new wxStaticText(this, -1, wxT("Station Location Name"));
+	st = new wxStaticText(this, -1, _("Station Location Name"));
 	sizer->Add(st, 0, wxALIGN_LEFT|wxTOP, 10);
 	newname = new wxTextCtrl(this, TQSL_ID_LOW+1, wxT(""), wxPoint(0, y), wxSize(control_width, -1));
 	sizer->Add(newname, 0, wxEXPAND);
 	newname->SetValue(parent->GetLocationName());
+	errlbl = new wxStaticText(this, -1, wxT(""), wxDefaultPosition, wxSize(control_width, -1),
+				wxALIGN_LEFT/*|wxST_NO_AUTORESIZE*/);
+	sizer->Add(errlbl, 0, wxTOP);
+	initialized = true;
 	AdjustPage(sizer, wxT("stnloc2.htm"));
 }
 
 bool
 TQSLWizFinalPage::TransferDataFromWindow() {
 	tqslTrace("TQSLWizFinalPage::TransferDataFromWindow");
-	if (validate())	 // Must be a "back"
+	validate();
+	if (valMsg.Len() > 0) // Must be a "back"
 		return true;
 	wxString s = newname->GetValue().Trim(true).Trim(false);
 	(reinterpret_cast<TQSLWizard *>(GetParent()))->SetLocationName(s);
 	return true;
 }
 
+void
+TQSLWizFinalPage::OnPageChanging(wxWizardEvent& ev) {
+	tqslTrace("TQSLWizFinalPage::OnPageChanging", "Direction=", ev.GetDirection());
+
+	validate();
+	if (valMsg.Len() > 0 && ev.GetDirection()) {
+		ev.Veto();
+		wxMessageBox(valMsg, _("Error"));
+	}
+}
+
 const char *
 TQSLWizFinalPage::validate() {
 	tqslTrace("TQSLWizFinalPage::validate");
+	if (!initialized) return 0;
 	wxString val = newname->GetValue().Trim(true).Trim(false);
-	const char *errmsg = 0;
-	val.Trim().Trim(false);
-	if (val == wxT(""))
-		errmsg = "Station name must be provided";
-	return errmsg;
+	valMsg = wxT("");
+
+	if (val == wxT("")) {
+		valMsg = wxGetTranslation(_("Station name must be provided"));
+	}
+	if (errlbl) errlbl->SetLabel(valMsg);
+	return 0;
 }
 
 TQSLWizFinalPage::~TQSLWizFinalPage() {
