@@ -1,6 +1,6 @@
 #!/bin/sh
 
-TQSLVER=`cat apps/tqslversion.ver`
+TQSLVER=`cat apps/tqslversion.ver|sed -e 's/\.0$//'`
 TQSLLIBPATH=`pwd`/src/libtqsllib.dylib
 WORKDIR=`mktemp -d /tmp/tqsl.XXXXX` || exit 1
 
@@ -22,6 +22,14 @@ do
     install_name_tool -change $TQSLLIBPATH @executable_path/libtqsllib.dylib $WORKDIR/TrustedQSL/$app.app/Contents/MacOS/$app
     cp src/config.xml $WORKDIR/TrustedQSL/$app.app/Contents/Resources
     cp apps/ca-bundle.crt $WORKDIR/TrustedQSL/$app.app/Contents/Resources
+    for lang in de es fr it ja pt
+    do
+	mkdir $WORKDIR/TrustedQSL/$app.app/Contents/Resources/$lang.lproj
+	cp apps/lang/$lang/tqslapp.mo $WORKDIR/TrustedQSL/$app.app/Contents/Resources/$lang.lproj
+	cp apps/lang/$lang/wxstd.mo $WORKDIR/TrustedQSL/$app.app/Contents/Resources/$lang.lproj
+    done
+# Make an empty 'en.lproj' folder so wx knows it's default
+    mkdir $WORKDIR/TrustedQSL/$app.app/Contents/Resources/en.lproj
 done
 
 /bin/echo "done"
